using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace Tutorial_4_1
{
    class GrfxWindowXNA
    {
        //XNA graphics device
        private GraphicsDevice m_XNA_GraphicsDevice;

        private EffectPool m_EffectPool;
        private BasicEffect m_BasicEffect;

        //This is our customer vertex structure XNA likes to recieve 
        //arrays of structs with vertex information. For simple, unlit
        //drawing; a vertex and color at the vertex will suffice.
        public struct DeviceVertexFormat
        {
            private Vector3 m_Position;
            private Color m_Color;

            //Format in which this type of vertex will be passed to the graphics hardware
            public static VertexElement[] VertexElements =
            {
                new VertexElement(0, 0, VertexElementFormat.Vector3, VertexElementMethod.Default, 
                                  VertexElementUsage.Position, 0),
                new VertexElement(0, sizeof(float)*3, VertexElementFormat.Color, VertexElementMethod.Default, 
                                  VertexElementUsage.Color, 0) 
            };

            //m_Position accessor and modifier
            public Vector3 Position
            {
                get
                {
                    return m_Position;
                }
                set
                {
                    m_Position = value;
                }
            }

            //m_Color accessor and modifier
            public Color Color
            {
                get
                {
                    return m_Color;
                }
                set
                {
                    m_Color = value;
                }
            }
        };

        //Replaces the PlaceHolder with an XNA graphics the GrfxWindowXNA 
        public void ReplaceFormControl(Control PlaceHolder)
        {
            Create_Graphics_Device(PlaceHolder.Handle, PlaceHolder.Width, PlaceHolder.Height);

            //Links the PlaceHolder's Paint Event with the GrfxWindowXNA's OnPaint event
            PlaceHolder.Paint += new PaintEventHandler(OnPaint);
        }

        //Clears the graphics device and draws the two rectangles to it
        private void OnPaint(object sender, EventArgs e)
        {
            //set the model matrix to identity
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.View = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;
            
            //taking parameters from the grfxPanel guarentees that you get the correct
            //aspect ratio for graphics rendering
            Panel grfxPanel = (Panel)sender;

            Matrix World2NDC = new Matrix();
            Vector3 scale = new Vector3(2f / grfxPanel.Width, 2f / grfxPanel.Height, 1f);
            Vector3 translate = new Vector3(-1f, -1f, 0f);
            World2NDC = Matrix.CreateScale(scale) * Matrix.CreateTranslation(translate);

            m_BasicEffect.View = World2NDC;
            m_BasicEffect.VertexColorEnabled = true;
            
            //set some of the renderstate's variables
            m_XNA_GraphicsDevice.RenderState.CullMode = CullMode.None;
            m_XNA_GraphicsDevice.RenderState.FillMode = FillMode.Solid;

            //clear the graphics device with a grey background color
            m_XNA_GraphicsDevice.Clear(new Color(new Vector4(.8f, .8f, .8f, 0f)));

            //begin scene drawing
            m_BasicEffect.Begin();
            m_BasicEffect.CurrentTechnique.Passes[0].Begin();

            //------draw the two rectangles------
            //create the rectangles vertices
            DeviceVertexFormat[] v = new DeviceVertexFormat[4];

            for (int i = 0; i < 4; i++)
            {
                v[i] = new DeviceVertexFormat();
                v[i].Color = new Color(new Vector4(.4f, .4f, .4f, 0f));
            }

            //large square
            v[0].Position = new Vector3(160f, 122f, 0f);
            v[1].Position = new Vector3(80f, 122f, 0f);
            v[2].Position = new Vector3(80f, 42f, 0f);
            v[3].Position = new Vector3(160f, 42f, 0f);
            m_XNA_GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleFan, v, 0, 2);

            //small square
            v[0].Position = new Vector3(160f, 122f, 0f);
            v[1].Position = new Vector3(210f, 122f, 0f);
            v[2].Position = new Vector3(210f, 172f, 0f);
            v[3].Position = new Vector3(160f, 172f, 0f);
            m_XNA_GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleFan, v, 0, 2);
            //------end of rectangle drawing------
            m_BasicEffect.CurrentTechnique.Passes[0].End();
            m_BasicEffect.End();

            //Show what we drew
            m_XNA_GraphicsDevice.Present();
        }

        //Creates the XNA graphics device if one doesn't already exist
        private void Create_Graphics_Device(IntPtr handle, int width, int height)
        {
            if (m_XNA_GraphicsDevice == null) //don't re-create the graphics device if it already exists
            {
                //define presentation parameters
                PresentationParameters pp = new PresentationParameters();
                pp.BackBufferCount = 1;
                pp.BackBufferFormat = SurfaceFormat.Unknown;
                pp.BackBufferWidth = width;
                pp.BackBufferHeight = height;
                pp.DeviceWindowHandle = handle;
                pp.IsFullScreen = false;
                pp.SwapEffect = SwapEffect.Discard;

                //create an XNA graphics device
                m_XNA_GraphicsDevice = new GraphicsDevice(GraphicsAdapter.DefaultAdapter, DeviceType.Hardware,
                    handle, pp);

                //check if graphics device was created
                Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

                m_XNA_GraphicsDevice.VertexDeclaration = new VertexDeclaration(m_XNA_GraphicsDevice, DeviceVertexFormat.VertexElements);

                m_EffectPool = new EffectPool();
                m_BasicEffect = new BasicEffect(m_XNA_GraphicsDevice, m_EffectPool);
            }
        }
    }
}
