using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib3;

namespace XNA_BallShoot
{
    class DrawAndMouseHandler : UWB_XNAWindowHandler
    {
        private UWBGL_UIWindow mWindow;

        public DrawAndMouseHandler()
        {
            mWindow = new UWBGL_UIWindow(true);
        }

        public bool Initialize(Control PlaceHolder)
        {
            if (!mWindow.ReplaceFormControl(PlaceHolder.Handle))
                return false;

            if (!mWindow.AttachHandler(this))
                return false;
            
            return true;
        }

        public override void DrawGraphics()
        {
            float height = 0;
            float width = 0;
            Program.getModel().getWorldSize(ref width, ref height);
            width = height * getAspectRatio();

            Matrix World2NDC = new Matrix();
            Vector3 scale = new Vector3(2f / width, 2f / height, 1f);
            Vector3 translate = new Vector3(-1f, -1f, 0f);
            World2NDC = Matrix.CreateScale(scale) * Matrix.CreateTranslation(translate);

            UWB_XNAGraphicsSystem.m_TheAPI.setViewMatrix(World2NDC);

            BeginScene();

            Program.getModel().drawModel();

            EndSceneAndShow();
        }

        public override void OnMouseButton(bool down, MouseEventArgs e)
        {
            int deviceX = 0, deviceY = 0;
            hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);

            string key = "";
            Keys modkey = Control.ModifierKeys;
            if (modkey == Keys.Control)
                key += "[CTRL]";
            if (modkey == Keys.Shift)
                key += "[SHIFT]";
            if (key.Length == 0)
                key += "[NONE]";

            string btn = "";
            switch (e.Button)
            {
                case MouseButtons.Left:
                    btn = "LMB";
                    if (down)
                    {
                        Program.getModel().createHeroBall((float)deviceX, (float)deviceY);
                    }
                    else
                        Program.getModel().addHeroBallToWorld();
                    break;
                case MouseButtons.Right:
                    btn = "RMB";
                    if(down)
                    {
                        Program.getModel().selectBallAt((float)deviceX, (float)deviceY);
                    }
                    break;
                case MouseButtons.Middle:
                    btn = "MMB";
                    break;
            }

            string msg = "Viewport: " + btn + ": Position: (" + deviceX.ToString() + " " + deviceY.ToString() + ") Keys: " + key;
            UWB_Utility.echoToStatusArea(msg);
        }

        public override void OnMouseMove(MouseEventArgs e)
        {
            int deviceX = 0, deviceY = 0;
            hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);

            string key = "";
            Keys modkey = Control.ModifierKeys;
            if (modkey == Keys.Control)
                key += "[CTRL]";
            if (modkey == Keys.Shift)
                key += "[SHIFT]";
            if (key.Length == 0)
                key += "[NONE]";

            string btn = "";
            switch (e.Button)
            {
                case MouseButtons.Left:
                    btn = "LMB";
                    Program.getModel().resizeHeroBall((float)deviceX, (float)deviceY);
                    break;
                case MouseButtons.Right:
                    btn = "RMB";
                    Program.getModel().moveSelectedBallTo((float)deviceX, (float)deviceY);
                    break;
                case MouseButtons.Middle:
                    btn = "MMB";
                    break;
            }

            string msg = "Viewport: " + btn + ": Position: (" + deviceX.ToString() + " " + deviceY.ToString() + ") Keys: " + key;
            UWB_Utility.echoToStatusArea(msg);
        }
    }
}
