using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib7;

namespace XNA_ObjectCollections
{
    public partial class XNA_ObjectCollections : Form
    {
        private DrawAndMouseHandler mView;
        private Timer mTimer;

        public XNA_ObjectCollections()
        {
            mView = new DrawAndMouseHandler();
            mTimer = new Timer();            
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Program.getModel().setWorldSize(PlaceHolder_Graphics.Width, PlaceHolder_Graphics.Height);
            mView.Initialize(PlaceHolder_Graphics);
           
            UWBGL_XNA_Lib7.UWB_Utility.mEchoTextBox = echoTextBox;

            //set our timer's interval in milliseconds and set the mTimer_Tick function to be
            //called every time it reaches that interval
            mTimer.Interval = 25;
            mTimer.Tick += new EventHandler(mTimer_Tick);
            mTimer.Start();

			sliderPointSize.ChangeRangeValues(0, 15, 2f);

			Program.getModel().setCurrentTypeToPoint();
			Program.getModel().setLevelOfDetail(UWBGL_XNA_Lib7.eLevelofDetail.lodHigh);
			Program.getModel().setShadeMode(UWBGL_XNA_Lib7.eShadeMode.smFlat);

            UWBGL_XNA_Lib7.UWB_Utility.echoToStatusArea("Form Load Complete");
        }

        private void Button_Quit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void mTimer_Tick(object sender, EventArgs e)
        {
            Program.getModel().updateSimulation();

            mView.DrawGraphics();
        }

		private void sliderPointSize_Scroll(object sender, EventArgs e)
		{
			float value = sliderPointSize.GetSliderValue();
			Program.getModel().setPointSize(value);
		}

		private void radioPoint_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().setCurrentTypeToPoint();
			setUI();
		}

		private void radioLine_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().setCurrentTypeToLine();
			setUI();
		}

		private void radioCircle_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().setCurrentTypeToCircle();
			setUI();
		}

		private void checkBoxShading_CheckedChanged(object sender, EventArgs e)
		{
			if(checkBoxShading.Checked)
				Program.getModel().setShadeMode(UWBGL_XNA_Lib7.eShadeMode.smGouraud);
			else
				Program.getModel().setShadeMode(UWBGL_XNA_Lib7.eShadeMode.smFlat);
		}

		private void checkBoxFillSolid_CheckedChanged(object sender, EventArgs e)
		{
			if(checkBoxFillSolid.Checked)
				Program.getModel().setFillMode(UWBGL_XNA_Lib7.eFillMode.fmSolid);
			else
				Program.getModel().setFillMode(UWBGL_XNA_Lib7.eFillMode.fmWireframe);
		}

		private void setUI()
		{
			if (Program.getModel().getShadeMode() == UWBGL_XNA_Lib7.eShadeMode.smGouraud)
				checkBoxShading.Checked = true;
			else
				checkBoxShading.Checked = false;

			if (Program.getModel().getFillMode() == UWBGL_XNA_Lib7.eFillMode.fmSolid)
				checkBoxFillSolid.Checked = true;
			else
				checkBoxFillSolid.Checked = false;

			sliderPointSize.SetSliderValue(Program.getModel().getPointSize());
		}

		private void radioButtonLodLow_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().setLevelOfDetail(UWBGL_XNA_Lib7.eLevelofDetail.lodLow);
		}

		private void radioButtonLodMedium_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().setLevelOfDetail(UWBGL_XNA_Lib7.eLevelofDetail.lodMed);
		}

		private void radioButtonLodHigh_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().setLevelOfDetail(UWBGL_XNA_Lib7.eLevelofDetail.lodHigh);
		}

		private void buttonShadingColor_Click_1(object sender, EventArgs e)
		{
			if (shadingColorDialog.ShowDialog() != DialogResult.Cancel)
			{
				Microsoft.Xna.Framework.Graphics.Color color = new Microsoft.Xna.Framework.Graphics.Color(
					shadingColorDialog.Color.R,
					shadingColorDialog.Color.G,
					shadingColorDialog.Color.B,
					shadingColorDialog.Color.A);

				Program.getModel().setShadingColor(color);

				string msg = "Set shading color to: (R=" + shadingColorDialog.Color.R +
							 " G=" + shadingColorDialog.Color.G +
							 " B=" + shadingColorDialog.Color.B + ")";
				UWB_Utility.echoToStatusArea(msg);
			}
		}

		private void buttonColor_Click_1(object sender, EventArgs e)
		{
			if (colorDialog.ShowDialog() != DialogResult.Cancel)
			{
				Microsoft.Xna.Framework.Graphics.Color color = new Microsoft.Xna.Framework.Graphics.Color(
					colorDialog.Color.R,
					colorDialog.Color.G,
					colorDialog.Color.B,
					colorDialog.Color.A);

				Program.getModel().setObjectColor(color);

				string msg = "Set draw color to: (R=" + shadingColorDialog.Color.R +
							 " G=" + shadingColorDialog.Color.G +
							 " B=" + shadingColorDialog.Color.B + ")";
				UWB_Utility.echoToStatusArea(msg);
			}
		}

		private void checkBoxShading_CheckedChanged_1(object sender, EventArgs e)
		{
			if (checkBoxShading.Checked)
				Program.getModel().setShadeMode(UWBGL_XNA_Lib7.eShadeMode.smGouraud);
			else
				Program.getModel().setShadeMode(UWBGL_XNA_Lib7.eShadeMode.smFlat);
		}

		private void checkBox1_CheckedChanged(object sender, EventArgs e)
		{
			Program.getModel().showBoundingVolumes(checkBoxShowBounds.Checked);
		}
    }
}
