using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib9;

namespace XNA_XFormInfo
{
	public class Model
	{
		private struct RectWithXForm
		{
			public UWB_PrimitiveRectangle mRect;
			public UWB_XFormInfo mXForm;
		};

		private RectWithXForm mRect1;
		private RectWithXForm mRect2;

		private UWB_Clock mStopWatch;

		private UWB_BoundingBox mWorldBounds;

		private UWB_XNADrawHelper mDrawHelper;

		public Model()
		{
			mRect1.mRect = new UWB_PrimitiveRectangle();
			mRect1.mXForm = new UWB_XFormInfo();
			mRect1.mRect.setCorners(Vector3.Zero, new Vector3(2, 3, 0));
			mRect1.mRect.setFlatColor(new Color(50, 0, 0));
			mRect2.mRect = new UWB_PrimitiveRectangle();
			mRect2.mXForm = new UWB_XFormInfo();
			mRect2.mRect.setCorners(new Vector3(5, 5, 0), new Vector3(8, 6, 0));
			mRect2.mRect.setFlatColor(new Color(0, 50, 0));

			mWorldBounds = new UWB_BoundingBox();
			mWorldBounds.setCorners(Vector3.Zero, new Vector3(12, 12, 0));
			mRect1.mXForm.setPivot(mRect1.mRect.getLocation());
			mRect2.mXForm.setPivot(mRect2.mRect.getLocation());

			mDrawHelper = new UWB_XNADrawHelper();
			mDrawHelper.initializeModelTransform();

			mStopWatch = new UWB_Clock();

			mWorldBounds.setCorners(Vector3.Zero, new Vector3(12, 12, 0));
		}

		public void updateSimulation()
		{
			//nothing to do here
		}

		public void setWorldAspectRatio(float width, float height)
		{
			if (height > 0)
			{
				float worldHeight = 13;
				float ar = width / height;
				float worldWidth = ar * worldHeight;
				mWorldBounds.setCorners(Vector3.Zero, new Vector3(worldWidth, worldHeight, 0));
			}
		}

		public void getWorldBounds(ref float width, ref float height)
		{
			width = mWorldBounds.width();
			height = mWorldBounds.height();
		}

		public UWB_BoundingBox getWorldBounds()
		{
			return mWorldBounds;
		}

		public void setTranslation(float x, float y)
		{
			mRect1.mXForm.setTranslation(new Vector3(x, y, 0));
			mRect2.mXForm.setTranslation(new Vector3(x, y, 0));
		}

		public void setScale(float x, float y)
		{
			mRect1.mXForm.setScale(new Vector3(x, y, 0));
		}

		public void setRotation(float radians)
		{
			mRect2.mXForm.setRotationRadians(radians);
		}

		public void drawGrid(UWB_DrawHelper drawHelper, UWB_BoundingBox bounds)
		{
			drawHelper.setShadeMode(eShadeMode.smFlat);
			drawHelper.setColor1(new Color(150, 150, 150));

			Vector3 minPoint = bounds.getMin();
			Vector3 maxPoint = bounds.getMax();

			float y = minPoint.Y;
			while (y < maxPoint.Y)
			{
				drawHelper.drawLine(new Vector3(minPoint.X, y, 0), new Vector3(maxPoint.X, y, 0));
				y += 1;
			}

			float x = minPoint.X;
			while (x < maxPoint.X)
			{
				drawHelper.drawLine(new Vector3(x, minPoint.Y, 0), new Vector3(x, maxPoint.Y, 0));
				x += 1;
			}
		}

		public void drawModel()
		{
			eLevelofDetail lod = mDrawHelper.getLod();

			mDrawHelper.initializeModelTransform();

			drawGrid(mDrawHelper, mWorldBounds);

			mDrawHelper.pushModelTransform();
			mRect1.mXForm.setupModelStack(mDrawHelper);
			mRect1.mRect.draw(lod, mDrawHelper);
			mRect1.mXForm.drawPivot(mDrawHelper, .1f);
			mDrawHelper.popModelTransform();

			mDrawHelper.pushModelTransform();
			mRect2.mXForm.setupModelStack(mDrawHelper);
			mRect2.mRect.draw(lod, mDrawHelper);
			mRect2.mXForm.drawPivot(mDrawHelper, .1f);
			mDrawHelper.popModelTransform();
		}
	}
}

